@@ -124,17 +124,14 @@ void ObexFtpDaemon::offlineMode()
         return;
     }
 
-    QHash<QString, ObexSession*>::const_iterator i = d->m_sessionMap.constBegin();
-    while (i != d->m_sessionMap.constEnd()) {
-        if (d->m_sessionMap[i.key()]) {
-            d->m_sessionMap[i.key()]->Disconnect().waitForFinished();
-            d->m_sessionMap[i.key()]->Close().waitForFinished();
-            delete d->m_sessionMap[i.key()];
-        }
-        d->m_sessionMap.remove(i.key());
-        ++i;
+    Q_FOREACH(ObexSession *session, d->m_sessionMap) {
+        session->Disconnect();
+        session->Close();
+        session->deleteLater();
     }
 
+    d->m_sessionMap.clear();
+
     delete d->m_manager;
     d->m_status = Private::Offline;
 }
@@ -313,6 +310,11 @@ void ObexFtpDaemon::SessionConnected(QDBusObjectPath path)
 
     QString address = getAddressFromSession(path.path());
 
+    if (address.isEmpty() || !d->m_sessionMap.contains(address)) {
+        kDebug() << "This seasson is from another process";
+        return;
+    }
+
     d->m_sessionMap[address]->setStatus(ObexSession::Connected);
 
     connect(d->m_sessionMap[address], SIGNAL(sessionTimeout()), this, SLOT(sessionDisconnected()));
@@ -332,20 +334,22 @@ void ObexFtpDaemon::SessionConnected(QDBusObjectPath path)
 void ObexFtpDaemon::SessionClosed(QDBusObjectPath path)
 {
     kDebug();
-    QHash<QString, ObexSession*>::const_iterator i = d->m_sessionMap.constBegin();
-    while (i != d->m_sessionMap.constEnd()) {
-        //If the session is connected, so not 0
+
+    QHashIterator<QString, ObexSession*> i(d->m_sessionMap);
+    while (i.hasNext()) {
+        i.next();
         if (i.value()->path() == path.path()) {
             kDebug() << "Removing : " << i.key();
             emit sessionClosed(i.key());
+
+            i.value()->deleteLater();
+
             d->m_sessionMap.remove(i.key());
-            delete i.value();
             return;
         }
-        ++i;
     }
 
-    kDebug() << "Attempt to remove a nto existing session";
+    kDebug() << "Attempt to remove a not existing session";
 }
 
 void ObexFtpDaemon::sessionDisconnected()
